==> teradataml-16.20.0.0-py3-none-any.whl: teradataml/analytics/Pack.py <==
#!/usr/bin/python
# ################################################################## 
# 
# Copyright 2018 Teradata. All rights reserved.
# TERADATA CONFIDENTIAL AND TRADE SECRET
# 
# Primary Owner: Pankaj Purandare (pankajvinod.purandare@teradata.com)
# Secondary Owner: Mounika Kotha (mounika.kotha@teradata.com)
# 
# Version: 1.1
# Function Version: 1.0
# 
# ################################################################## 

from teradataml.common.wrapper_utils import AnalyticsWrapperUtils
from teradataml.common.utils import UtilFuncs
from teradataml.context.context import *
from teradataml.dataframe.dataframe import DataFrame
from teradataml.common.aed_utils import AedUtils
from teradataml.analytics.analytic_query_generator import AnalyticQueryGenerator
from teradataml.common.exceptions import TeradataMlException
from teradataml.common.messages import Messages
from teradataml.common.messagecodes import MessageCodes
from teradataml.common.constants import TeradataConstants
from teradataml.dataframe.dataframe_utils import DataFrameUtils as df_utils
from teradataml.options.display import display

class Pack:
    
    def __init__(self,
        data = None,
        input_columns = None,
        output_column = None,
        delimiter = ",",
        include_column_name = True) :
        """
        DESCRIPTION:
            The Pack function packs data from multiple input columns into a 
            single column. The packed column has a virtual column for each input 
            column. By default, virtual columns are separated by commas and each 
            virtual column value is labeled with its column name.
        
        
        PARAMETERS:
            data:
                Required Argument.
                The TeradataML DataFrame containing the input attributes.
            
            input_columns:
                Optional Argument.
                Specifies the names of the input columns to pack into a single output 
                column. These names become the column names of the virtual columns. 
                By default, all input TeradataML DataFrame columns are packed into a 
                single output column. If you specify this argument, but do not 
                specify all input TeradataML DataFrame columns, the function copies 
                the unspecified input table columns to the output table.
            
            output_column:
                Required Argument.
                Specifies the name to give to the packed output column.
            
            delimiter:
                Optional Argument.
                Specifies the delimiter (a string) that separates the virtual columns 
                in the packed data. The default delimiter is comma (,).
                Default Value: ","
            
            include_column_name:
                Optional Argument.
                Specifies whether to label each virtual column value with its column 
                name (making the virtual column "input_column:value"). 
                Default Value: True
        
        RETURNS:
            Instance of Pack.
            Output TeradataML DataFrames can be accessed using attribute
            references, such as PackObj.<attribute_name>.
            Output TeradataML DataFrame attribute name is:
                result
        
        
        RAISES:
            TeradataMlException
        
        
        EXAMPLES:
            # Create TeradataML DataFrame objects
            ville_temperature = DataFrame.from_table("ville_temperature")

            # Example1 - 
            pack_out = Pack(data=ville_temperature,
                            input_columns=['city','state','period','temp_f'],
                            output_column='packed_data',
                            delimiter=',',
                            include_column_name=True)
            
            # Print the results
            pack_out.result
        
        """
        self.data  = data 
        self.input_columns  = input_columns 
        self.output_column  = output_column 
        self.delimiter  = delimiter 
        self.include_column_name  =  include_column_name 
        
        # Create TeradataPyWrapperUtils instance which contains validation functions.
        self.__awu = AnalyticsWrapperUtils()
        self.__aed_utils = AedUtils()
        
        # Create argument information matrix to do parameter checking
        self.__arg_info_matrix = []
        self.__arg_info_matrix.append(["data", self.data, False, "DataFrame"])
        self.__arg_info_matrix.append(["input_columns", self.input_columns, True, "str"])
        self.__arg_info_matrix.append(["output_column", self.output_column, False, "str"])
        self.__arg_info_matrix.append(["delimiter", self.delimiter, True, "str"])
        self.__arg_info_matrix.append(["include_column_name", self.include_column_name, True, "bool"])
        
        # Perform the function validations
        self.__validate()
        # Generate the ML query
        self.__form_tdml_query()
        # Execute ML query
        self.__execute()
        
    def __validate(self) :
        """
        Function to validate sqlmr function arguments to verify missing 
        arguments, input argument and table types. Also processes the 
        argument values.
        """
        
        # Make sure that a non-NULL value has been supplied for all mandatory arguments
        self.__awu._validate_missing_required_arguments(self.__arg_info_matrix)
        
        # Make sure that a non-NULL value has been supplied correct type of argument
        self.__awu._validate_argument_types(self.__arg_info_matrix)
        
        # Check to make sure input table types are strings or data frame objects or of valid type.
        self.__awu._validate_input_table_datatype(self.data, "data", None)
        
        # Check whether the input columns passed to the argument are not empty.
        # Also check whether the input columns passed to the argument valid or not.
        self.__awu._validate_input_columns_not_empty(self.input_columns, "input_columns")
        self.__awu._validate_dataframe_has_argument_columns(self.input_columns, "input_columns", self.data, "data")
        
        self.__awu._validate_input_columns_not_empty(self.output_column, "output_column")
        
        
        
    def __form_tdml_query(self) :
        """
        Function to generate the analytical function queries. The function defines variables and list of arguments required to form the query.
        """
        
        # Output table arguments list
        self.__func_output_args_sql_names = []
        self.__func_output_args = []
        
        # Generate lists for rest of the function arguments
        self.__func_other_arg_sql_names = []
        self.__func_other_args = []
        self.__func_other_arg_json_datatypes = []
        
        if self.input_columns is not None:
            self.__func_other_arg_sql_names.append("InputColumns")
            self.__func_other_args.append(UtilFuncs._teradata_collapse_arglist(self.input_columns,"'"))
            self.__func_other_arg_json_datatypes.append("COLUMNS")
        
        self.__func_other_arg_sql_names.append("OutputColumn")
        self.__func_other_args.append(UtilFuncs._teradata_collapse_arglist(self.output_column,"'"))
        self.__func_other_arg_json_datatypes.append("STRING")
        
        if self.delimiter is not None and self.delimiter != ",":
            self.__func_other_arg_sql_names.append("Delimiter")
            self.__func_other_args.append(UtilFuncs._teradata_collapse_arglist(self.delimiter,"'"))
            self.__func_other_arg_json_datatypes.append("STRING")
        
        if self. include_column_name is not None and self. include_column_name != True:
            self.__func_other_arg_sql_names.append("IncludeColumnName")
            self.__func_other_args.append(UtilFuncs._teradata_collapse_arglist(self. include_column_name,"'"))
            self.__func_other_arg_json_datatypes.append("BOOLEAN")
        
        
        # Declare empty lists to hold input table information.
        self.__func_input_arg_sql_names = []
        self.__func_input_table_view_query = []
        self.__func_input_dataframe_type = []
        self.__func_input_distribution = []
        self.__func_input_partition_by_cols = []
        self.__func_input_order_by_cols = []
        
        # Process data
        self.__table_ref = self.__awu._teradata_on_clause_from_dataframe(self.data, False)
        self.__func_input_distribution.append("FACT")
        self.__func_input_arg_sql_names.append("input")
        self.__func_input_table_view_query.append(self.__table_ref["ref"])
        self.__func_input_dataframe_type.append(self.__table_ref["ref_type"])
        self.__func_input_partition_by_cols.append("ANY")
        self.__func_input_order_by_cols.append("NA_character_")
        
        function_name = "Pack"
        # Create instance to generate SQLMR.
        aqg_obj = AnalyticQueryGenerator(function_name 
                ,self.__func_input_arg_sql_names 
                ,self.__func_input_table_view_query 
                ,self.__func_input_dataframe_type 
                ,self.__func_input_distribution 
                ,self.__func_input_partition_by_cols 
                ,self.__func_input_order_by_cols 
                ,self.__func_other_arg_sql_names 
                ,self.__func_other_args 
                ,self.__func_other_arg_json_datatypes 
                ,self.__func_output_args_sql_names 
                ,self.__func_output_args)
        # Invoke call to SQL-MR generation.
        self.sqlmr_query = aqg_obj._gen_sqlmr_select_stmt_sql()
        
        # Print SQL-MR query if requested to do so.
        if display.print_sqlmr_query:
            print(self.sqlmr_query)
        
    def __execute(self) :
        """
        Function to execute SQL-MR queries. 
        Creates dataframes for the required SQL-MR outputs.
        """
        # Generate STDOUT table name and add it to the output table list.
        sqlmr_stdout_temp_tablename = UtilFuncs._generate_temp_table_name(prefix = "td_sqlmr_out_", use_default_database = True, gc_on_quit = True, quote=False)
        try:
            UtilFuncs._create_view(sqlmr_stdout_temp_tablename, self.sqlmr_query)
        except Exception as emsg:
            raise TeradataMlException(Messages.get_message(MessageCodes.TDMLDF_EXEC_SQL_FAILED, str(emsg)), MessageCodes.TDMLDF_EXEC_SQL_FAILED)
        
        # Update output table data frames.
        self._mlresults = []
        self.result = self.__awu._create_data_set_object(df_input=UtilFuncs._extract_table_name(sqlmr_stdout_temp_tablename), source_type="table", database_name=UtilFuncs._extract_db_name(sqlmr_stdout_temp_tablename))
        self._mlresults.append(self.result)
        
    def __repr__(self) :
        """
        Returns the string representation for a Pack class instance.
        """
        repr_string="############ STDOUT Output ############"
        repr_string = "{}\n\n{}".format(repr_string,self.result)
        return repr_string

==> teradataml-16.20.0.1.post0-py3-none-any.whl: teradataml/analytics/Pack.py <==
#!/usr/bin/python
# ################################################################## 
# 
# Copyright 2018 Teradata. All rights reserved.
# TERADATA CONFIDENTIAL AND TRADE SECRET
# 
# Primary Owner: Pankaj Purandare (pankajvinod.purandare@teradata.com)
# Secondary Owner: Mounika Kotha (mounika.kotha@teradata.com)
# 
# Version: 1.1
# Function Version: 1.0
# 
# ################################################################## 

from teradataml.common.wrapper_utils import AnalyticsWrapperUtils
from teradataml.common.utils import UtilFuncs
from teradataml.context.context import *
from teradataml.dataframe.dataframe import DataFrame
from teradataml.common.aed_utils import AedUtils
from teradataml.analytics.analytic_query_generator import AnalyticQueryGenerator
from teradataml.common.exceptions import TeradataMlException
from teradataml.common.messages import Messages
from teradataml.common.messagecodes import MessageCodes
from teradataml.common.constants import TeradataConstants
from teradataml.dataframe.dataframe_utils import DataFrameUtils as df_utils
from teradataml.options.display import display
from teradataml.common.utils import package_deprecation

@package_deprecation('16.20.00.01', 'teradataml.analytics.sqle')
class Pack:
    
    def __init__(self,
        data = None,
        input_columns = None,
        output_column = None,
        delimiter = ",",
        include_column_name = True) :
        """
        DESCRIPTION:
            The Pack function packs data from multiple input columns into a 
            single column. The packed column has a virtual column for each input 
            column. By default, virtual columns are separated by commas and each 
            virtual column value is labeled with its column name.
        
        
        PARAMETERS:
            data:
                Required Argument.
                The teradataml DataFrame containing the input attributes.
            
            input_columns:
                Optional Argument.
                Specifies the names of the input columns to pack into a single output 
                column. These names become the column names of the virtual columns. 
                By default, all input teradataml DataFrame columns are packed into a 
                single output column. If you specify this argument, but do not 
                specify all input teradataml DataFrame columns, the function copies 
                the unspecified input table columns to the output table.
                Types: str OR list of Strings (str)
            
            output_column:
                Required Argument.
                Specifies the name to give to the packed output column.
                Types: str
            
            delimiter:
                Optional Argument.
                Specifies the delimiter (a string) that separates the virtual columns 
                in the packed data. The default delimiter is comma (,).
                Default Value: ","
                Types: str
            
            include_column_name:
                Optional Argument.
                Specifies whether to label each virtual column value with its column 
                name (making the virtual column "input_column:value"). 
                Default Value: True
                Types: bool
        
        RETURNS:
            Instance of Pack.
            Output teradataml DataFrames can be accessed using attribute
            references, such as PackObj.<attribute_name>.
            Output teradataml DataFrame attribute name is:
                result
        
        
        RAISES:
            TeradataMlException
        
        
        EXAMPLES:
            # Load the data to run the example.
            load_example_data("Pack", "ville_temperature")
            
            # Create teradataml DataFrame objects
            ville_temperature = DataFrame.from_table("ville_temperature")

            # Example1 - 
            pack_out = Pack(data=ville_temperature,
                            input_columns=['city','state','period','temp_f'],
                            output_column='packed_data',
                            delimiter=',',
                            include_column_name=True)
            
            # Print the results
            pack_out.result
        
        """
        self.data  = data 
        self.input_columns  = input_columns 
        self.output_column  = output_column 
        self.delimiter  = delimiter 
        self.include_column_name  =  include_column_name 
        
        # Create TeradataPyWrapperUtils instance which contains validation functions.
        self.__awu = AnalyticsWrapperUtils()
        self.__aed_utils = AedUtils()
        
        # Create argument information matrix to do parameter checking
        self.__arg_info_matrix = []
        self.__arg_info_matrix.append(["data", self.data, False, "DataFrame"])
        self.__arg_info_matrix.append(["input_columns", self.input_columns, True, "str"])
        self.__arg_info_matrix.append(["output_column", self.output_column, False, "str"])
        self.__arg_info_matrix.append(["delimiter", self.delimiter, True, "str"])
        self.__arg_info_matrix.append(["include_column_name", self.include_column_name, True, "bool"])
        
        # Perform the function validations
        self.__validate()
        # Generate the ML query
        self.__form_tdml_query()
        # Execute ML query
        self.__execute()
        
    def __validate(self) :
        """
        Function to validate sqlmr function arguments to verify missing 
        arguments, input argument and table types. Also processes the 
        argument values.
        """
        
        # Make sure that a non-NULL value has been supplied for all mandatory arguments
        self.__awu._validate_missing_required_arguments(self.__arg_info_matrix)
        
        # Make sure that a non-NULL value has been supplied correct type of argument
        self.__awu._validate_argument_types(self.__arg_info_matrix)
        
        # Check to make sure input table types are strings or data frame objects or of valid type.
        self.__awu._validate_input_table_datatype(self.data, "data", None)
        
        # Check whether the input columns passed to the argument are not empty.
        # Also check whether the input columns passed to the argument valid or not.
        self.__awu._validate_input_columns_not_empty(self.input_columns, "input_columns")
        self.__awu._validate_dataframe_has_argument_columns(self.input_columns, "input_columns", self.data, "data")
        
        self.__awu._validate_input_columns_not_empty(self.output_column, "output_column")
        
        
        
    def __form_tdml_query(self) :
        """
        Function to generate the analytical function queries. The function defines variables and list of arguments required to form the query.
        """
        
        # Output table arguments list
        self.__func_output_args_sql_names = []
        self.__func_output_args = []
        
        # Generate lists for rest of the function arguments
        self.__func_other_arg_sql_names = []
        self.__func_other_args = []
        self.__func_other_arg_json_datatypes = []
        
        if self.input_columns is not None:
            self.__func_other_arg_sql_names.append("InputColumns")
            self.__func_other_args.append(UtilFuncs._teradata_collapse_arglist(self.input_columns,"'"))
            self.__func_other_arg_json_datatypes.append("COLUMNS")
        
        self.__func_other_arg_sql_names.append("OutputColumn")
        self.__func_other_args.append(UtilFuncs._teradata_collapse_arglist(self.output_column,"'"))
        self.__func_other_arg_json_datatypes.append("STRING")
        
        if self.delimiter is not None and self.delimiter != ",":
            self.__func_other_arg_sql_names.append("Delimiter")
            self.__func_other_args.append(UtilFuncs._teradata_collapse_arglist(self.delimiter,"'"))
            self.__func_other_arg_json_datatypes.append("STRING")
        
        if self. include_column_name is not None and self. include_column_name != True:
            self.__func_other_arg_sql_names.append("IncludeColumnName")
            self.__func_other_args.append(UtilFuncs._teradata_collapse_arglist(self. include_column_name,"'"))
            self.__func_other_arg_json_datatypes.append("BOOLEAN")
        
        
        # Declare empty lists to hold input table information.
        self.__func_input_arg_sql_names = []
        self.__func_input_table_view_query = []
        self.__func_input_dataframe_type = []
        self.__func_input_distribution = []
        self.__func_input_partition_by_cols = []
        self.__func_input_order_by_cols = []
        
        # Process data
        self.__table_ref = self.__awu._teradata_on_clause_from_dataframe(self.data, False)
        self.__func_input_distribution.append("FACT")
        self.__func_input_arg_sql_names.append("input")
        self.__func_input_table_view_query.append(self.__table_ref["ref"])
        self.__func_input_dataframe_type.append(self.__table_ref["ref_type"])
        self.__func_input_partition_by_cols.append("ANY")
        self.__func_input_order_by_cols.append("NA_character_")
        
        function_name = "Pack"
        # Create instance to generate SQLMR.
        aqg_obj = AnalyticQueryGenerator(function_name 
                ,self.__func_input_arg_sql_names 
                ,self.__func_input_table_view_query 
                ,self.__func_input_dataframe_type 
                ,self.__func_input_distribution 
                ,self.__func_input_partition_by_cols 
                ,self.__func_input_order_by_cols 
                ,self.__func_other_arg_sql_names 
                ,self.__func_other_args 
                ,self.__func_other_arg_json_datatypes 
                ,self.__func_output_args_sql_names 
                ,self.__func_output_args)
        # Invoke call to SQL-MR generation.
        self.sqlmr_query = aqg_obj._gen_sqlmr_select_stmt_sql()
        
        # Print SQL-MR query if requested to do so.
        if display.print_sqlmr_query:
            print(self.sqlmr_query)
        
    def __execute(self) :
        """
        Function to execute SQL-MR queries. 
        Creates dataframes for the required SQL-MR outputs.
        """
        # Generate STDOUT table name and add it to the output table list.
        sqlmr_stdout_temp_tablename = UtilFuncs._generate_temp_table_name(prefix = "td_sqlmr_out_", use_default_database = True, gc_on_quit = True, quote=False)
        try:
            UtilFuncs._create_view(sqlmr_stdout_temp_tablename, self.sqlmr_query)
        except Exception as emsg:
            raise TeradataMlException(Messages.get_message(MessageCodes.TDMLDF_EXEC_SQL_FAILED, str(emsg)), MessageCodes.TDMLDF_EXEC_SQL_FAILED)
        
        # Update output table data frames.
        self._mlresults = []
        self.result = self.__awu._create_data_set_object(df_input=UtilFuncs._extract_table_name(sqlmr_stdout_temp_tablename), source_type="table", database_name=UtilFuncs._extract_db_name(sqlmr_stdout_temp_tablename))
        self._mlresults.append(self.result)
        
    def __repr__(self) :
        """
        Returns the string representation for a Pack class instance.
        """
        repr_string="############ STDOUT Output ############"
        repr_string = "{}\n\n{}".format(repr_string,self.result)
        return repr_string
